@@ -21,7 +21,8 @@ class Server(object):
                            is stored in an in-memory structure, so the use of
                            multiple connected servers is not possible.
     :param logger: To enable logging set to ``True`` or pass a logger object to
-                   use. To disable logging set to ``False``.
+                   use. To disable logging set to ``False``. The default is
+                   ``False``.
     :param binary: ``True`` to support binary payloads, ``False`` to treat all
                    payloads as text. On Python 2, if this is set to ``True``,
                    ``unicode`` values are treated as text, and ``str`` and
@@ -48,26 +49,32 @@ class Server(object):
                        The first async mode that has all its dependencies
                        installed is then one that is chosen.
     :param ping_timeout: The time in seconds that the client waits for the
-                         server to respond before disconnecting.
+                         server to respond before disconnecting. The default
+                         is 60 seconds.
     :param ping_interval: The interval in seconds at which the client pings
-                          the server.
+                          the server. The default is 25 seconds.
     :param max_http_buffer_size: The maximum size of a message when using the
-                                 polling transport.
-    :param allow_upgrades: Whether to allow transport upgrades or not.
+                                 polling transport. The default is 100,000,000
+                                 bytes.
+    :param allow_upgrades: Whether to allow transport upgrades or not. The
+                           default is ``True``.
     :param http_compression: Whether to compress packages when using the
-                             polling transport.
+                             polling transport. The default is ``True``.
     :param compression_threshold: Only compress messages when their byte size
-                                  is greater than this value.
+                                  is greater than this value. The default is
+                                  1024 bytes.
     :param cookie: Name of the HTTP cookie that contains the client session
                    id. If set to ``None``, a cookie is not sent to the client.
+                   The default is ``'io'``.
     :param cors_allowed_origins: List of origins that are allowed to connect
                                  to this server. All origins are allowed by
                                  default.
     :param cors_credentials: Whether credentials (cookies, authentication) are
-                             allowed in requests to this server.
+                             allowed in requests to this server. The default is
+                             ``True``.
     :param engineio_logger: To enable Engine.IO logging set to ``True`` or pass
                             a logger object to use. To disable logging set to
-                            ``False``.
+                            ``False``. The default is ``False``.
     """
     def __init__(self, client_manager=None, logger=False, binary=False,
                  json=None, async_handlers=False, **kwargs):
@@ -79,7 +86,7 @@ class Server(object):
             packet.Packet.json = json
             engineio_options['json'] = json
         engineio_options['async_handlers'] = False
-        self.eio = engineio.Server(**engineio_options)
+        self.eio = self._engineio_server_class()(**engineio_options)
         self.eio.on('connect', self._handle_eio_connect)
         self.eio.on('message', self._handle_eio_message)
         self.eio.on('disconnect', self._handle_eio_disconnect)
@@ -106,12 +113,16 @@ class Server(object):
         if client_manager is None:
             client_manager = base_manager.BaseManager()
         self.manager = client_manager
+        self.manager.set_server(self)
         self.manager_initialized = False
 
         self.async_handlers = async_handlers
 
         self.async_mode = self.eio.async_mode
 
+    def is_asyncio_based(self):
+        return False
+
     def on(self, event, handler=None, namespace=None):
         """Register an event handler.
 
@@ -149,8 +160,6 @@ class Server(object):
         client's acknowledgement callback function if it exists. The
         ``'disconnect'`` handler does not take a second argument.
         """
-        if '-' in event:
-            raise ValueError('event names cannot contain hypens')
         namespace = namespace or '/'
 
         def set_handler(handler):
@@ -172,6 +181,8 @@ class Server(object):
         """
         if not isinstance(namespace_handler, namespace.Namespace):
             raise ValueError('Not a namespace instance')
+        if self.is_asyncio_based() != namespace_handler.is_asyncio_based():
+            raise ValueError('Not a valid namespace class for this server')
         namespace_handler._set_server(self)
         self.namespace_handlers[namespace_handler.namespace] = \
             namespace_handler
@@ -346,9 +357,6 @@ class Server(object):
         This function returns the HTTP response body to deliver to the client
         as a byte sequence.
         """
-        if not self.manager_initialized:
-            self.manager_initialized = True
-            self.manager.initialize(self)
         return self.eio.handle_request(environ, start_response)
 
     def start_background_task(self, target, *args, **kwargs):
@@ -414,6 +422,8 @@ class Server(object):
             self.manager.disconnect(sid, namespace)
             self._send_packet(sid, packet.Packet(packet.ERROR,
                                                  namespace=namespace))
+            if sid in self.environ:  # pragma: no cover
+                del self.environ[sid]
             return False
         else:
             self._send_packet(sid, packet.Packet(packet.CONNECT,
@@ -486,6 +496,9 @@ class Server(object):
 
     def _handle_eio_connect(self, sid, environ):
         """Handle the Engine.IO connection event."""
+        if not self.manager_initialized:
+            self.manager_initialized = True
+            self.manager.initialize()
         self.environ[sid] = environ
         return self._handle_connect(sid, '/')
 
@@ -520,3 +533,6 @@ class Server(object):
     def _handle_eio_disconnect(self, sid):
         """Handle Engine.IO disconnect event."""
         self._handle_disconnect(sid, '/')
+
+    def _engineio_server_class(self):
+        return engineio.Server
